/**
 * @file
 * Provides base JS for gridstack.
 */
(function ($, Drupal, drupalSettings) {

  "use strict";

  /**
   * Provides base JS for gridstack.
   *
   * @type {{attach: Drupal.initEditableGridstack.attach}}
   */
  Drupal.behaviors.initEditableGridstack = {
    attach: function (context, settings) {
      // Attach additional options for a behavior.
      $('.paragraphs-behavior', context).once('initEditableGridstack').each(function () {
        // Attach additional options for a behavior.
        let $wrapper = $(this);
        // Get textarea and select objects.
        let active_breakpoint = drupalSettings.paragraphs_gridstack.active_breakpoint;

        let $textarea = $('textarea.' + active_breakpoint + '_storage', $wrapper);
        let $select = $('select.' + active_breakpoint + '_columns', $wrapper);

        makeGridstack($wrapper, $textarea, $select, [
          'gridstack_wrapper',
          'gridstack-wrapper-' + active_breakpoint,
        ]);

        // Control switching of display modes.
        $('input.paragraphs-gridstack-action', $wrapper).once().click(function (event) {
          event.preventDefault();

          let $action = $(this);
          let breakpoint_class = $action.data('breakpoint-class');

          // Update items, which are active.
          $('.paragraphs-gridstack-settings, .paragraphs-gridstack-action').removeClass('active');
          $('[data-breakpoint-class=' + breakpoint_class + ']').addClass('active');

          // Find related textarea and select.
          let $textarea = $('textarea[data-setting-breakpoint=' + breakpoint_class + '][data-setting-type="storage"]');
          let $select = $('select[data-setting-breakpoint=' + breakpoint_class + ']');

          $('div[data-gridstack-meta="true"]', $wrapper).remove();
          makeGridstack($wrapper, $textarea, $select, [
            'gridstack_wrapper',
            'gridstack-wrapper-' + active_breakpoint,
          ]);
        });
      });
    }
  }

  /**
   * Functionality for actions.
   *
   * @todo disable buttons in case of no previous or template data.
   *
   * @type {{attach: Drupal.initEditableGridstackActions.attach}}
   */
  Drupal.behaviors.initEditableGridstackActions = {
    attach: function (context, settings) {
      // Attach additional options for a behavior.
      $('.paragraphs-behavior', context).once('initEditableGridstackActions').each(function () {
        // Attach additional options for a behavior.
        let $wrapper = $(this);

        $('#pg-action-restore', $wrapper).once().click(function (event) {
          event.preventDefault();
          $(this).addClass('active');
          $('[class*=".storage"]' + '.active')[0].value = $('[class*=".previous"]' + '.active')[0].textContent;
        })

        // @todo complete this feature.
        $('#pg-action-set-by-template', $wrapper).once().click(function (event) {
          event.preventDefault();
          $(this).addClass('active');
          $('[class*=".storage"]' + '.active')[0].value = $('[class*=".template"]' + '.active')[0].textContent;
        })
      });
    }
  }

  /**
   * Prepare gridstack widget.
   *
   * @todo Complete this function.
   *
   * @param $wrapper
   *   Wrapper object.
   * @param $textarea
   *   Textarea object.
   * @param $select
   *   Select object.
   * @param wrapper_classes
   *   Wrapper classes.
   */
  function makeGridstack($wrapper, $textarea, $select, wrapper_classes) {
    $wrapper.append('<div data-gridstack-meta="true"><div></div></div>');
    let $div_wrapper = $('div[data-gridstack-meta="true"]', $wrapper);
    $div_wrapper.addClass(['gridstack_wrapper', ...wrapper_classes].join(' '));

    let $div = $('div', $div_wrapper);
    let drupal_selector = $textarea.data('drupal-selector');

    // Class grid-stack is rudiment from the previous version of the library.
    $div.addClass('grid-stack gridstack gridstack-' + drupal_selector);

    let items = [];

    try {
      items = JSON.parse($textarea.val());
    }
    catch (exception) {
      console.error('JSON parsing error were found in gridstack data.', exception);
    }

    let gridstack_optionset = drupalSettings.paragraphs_gridstack.gridstack_optionset;

    let $grid = GridStack.init({
      disableOneColumnMode: true,
      float: gridstack_optionset.float,
      // @todo Use options from the settings.
      //       allow lock
      //       rtl support
      //       no resize/no move per element
    },  `.gridstack-${drupal_selector}`);

    // @todo probably this is not needed in this function.
    // According to docs of the library:
    //  - this method needed to enable drag and drop and resize,
    //  - this method is shortcut for enableMove(true) and enableResize(true).
    $grid.enableMove(true);
    $grid.enableResize(true);

    $grid.column(checkColumns($select), 'moveScale');

    $select.change(() => {
      columnsChanged($select);
    });

    let $parent = $wrapper.parent();
    let $container = $parent.find('.paragraphs-nested tbody');

    // @todo get items from the backend.
    $('tr', $container).each(function (index) {
      let node = items[index] ?? {};

      if (!items[index]) {
        node = {
          id: index,
          w: 10,
          h: 10,
        };
      }

      $grid.addWidget(node);
    });

    // Run batch updates, commit changes and update DOM.
    $grid
        .batchUpdate()
        .commit();

    columnsChanged($select);

    $grid.on('added removed change', function (e, items) {
      writeGrid($grid, $textarea);
    });

    /**
     * Check columns count.
     *
     * @param $select
     *   Select object.
     *
     * @returns {number}
     */
    function checkColumns($select){
      let column_id = $select.val();
      let libraries_data = drupalSettings.paragraphs_gridstack.libraries_data;

      if (!libraries_data) {
        // Display error message about missing library.
        console.error('Missing defined gridstack libraries.');
      }

      return  libraries_data[column_id]['columns'];
    }

    /**
     * Change columns count.
     *
     * @param $select
     *   Select object.
     */
    function columnsChanged($select) {
      let columns = checkColumns($select);
      let $container = $div_wrapper.width() > 0 ? $div_wrapper : $wrapper;
      let height = $container.width() / columns;

      writeGrid($grid, $textarea);

      $grid.column(columns, 'moveScale');
      $grid.cellHeight(Math.floor(height));
      $grid.load(JSON.parse($textarea.val()));
    }

    /**
     * Write grid data to textarea.
     *
     * @param $grid
     *   Gridstack object.
     * @param $textarea
     *   Textarea object.
     */
    function writeGrid($grid, $textarea) {
      let data = [];
      $grid.engine.nodes.forEach(function (node) {
        // @todo use paragraph id.
        data.push({
          x: node.x,
          y: node.y,
          w: node.w,
          h: node.h,
          id: node.id,
        });
      });

      data.sort(function (a, b) {
        return a.id - b.id;
      });

      $textarea.val(JSON.stringify(data, null, '  '));
    }
  }
}(jQuery, Drupal, drupalSettings));
